function [fac,lam,eigval,ssr] = pc_factor(x,k)
% -------------------------------------------------------------------------
% Translation of SW's pc_factor()
% -------------------------------------------------------------------------
% /* Compute principal components estimates of factor model
%
%    Input:
%     x = txn matrix
%     k = number of factors
%
%     Model
%
%     x(it) = lam(i)'f(t) + u(it)
%
%     or
%
%     x = f*lam' + u
%
%
%     Output:
%
%     f = txk matrix of factors
%     lam = n*k matrix of factor loadings
%     eigval = kx1 vector of eigenvalues of x*x' (or x'x) ...
%     ssr = sum of squared u's
%
%     Normalization:
%      f is normalized so that each column has std dev = 1, thus F'F = t*I(k)
%
%     Calculation note:
%      Calculations are speeded by using the smaller of x*x' or x'x
% */
% -------------------------------------------------------------------------
[t,n] = size(x);
if k>0 % # of factors must be positive number
    
    if n<t % Calculations are speeded by using the smaller of x*x' or x'x
        xx = x'*x;
        [ve, va] = eig(xx); %ve:eigvector, va: eigvalue
        ve  = fliplr(ve);
        va  = flipud(diag(va));
        eigval = va(1:k);
        lam = ve(:,1:k);
        fac = x*lam;
    else
        xx = x*x';
        [ve, va] = eig(xx); %ve:eigvector, va: eigvalue
        ve  = fliplr(ve);
        va  = flipud(diag(va));
        eigval = va(1:k);
        fac = ve(:,1:k);
        lam = x'*fac;
    end
    % normalize
    sfac = sqrt(mean(fac.^2));
    fac  = fac./repmat(sfac,size(fac,1),1);
    lam  = (x'*fac)/t;
    ssr  = sum(va)-sum(eigval);
    
else % if # of fac smaller than 1, turn NaN
    ssr    = sum(sum(x.^2));
    lam    = NaN;
    eigval = NaN;
    fac    = NaN;
end











