% Factor estimation with a new identification strategy
% Date: 7/15/2013

% INPUT
% X        : data
% T1       : pre-break
% T2       : post-break
% r1       : # of factors (pre-break)
% r2       : # of factors (post-break)
% xcatcode : category code (should be in the same order as X)
% xcatname : category names

% SOME INPUTS
T1 = 250;
T2 = T-T1;

r1 = 4;
r2 = 6;

% catcode names
xcatname = { ...
'NIPA' ;...    %1'NIPA'                   
'IP  ' ;...    %2'IP'                     
'Emp ' ;...    %3'Emp/Unemp'              
'HSS ' ;...    %4'Hous.Starts'            
'Ord ' ;...    %5'Orders/Inv/Sales'       
'Pri ' ;...    %6'Prices'                 
'Prod' ;...    %7'EmpEarning/Productivity'
'IntR' ;...    %8'Int. Rates'             
'Mon ' ;...    %9'Money&Credit'           
'StPr' ;...    %10'StockPrices&Wealth'     
'HoPr' ;...    %11'Housing Prices'         
'ExR ' ;...    %12'Exch.Rates'             
'' ;...    %''                       
'' ;...    %'Other'                        
};


%% preliminary
% - sort data in xcatcode order
[sXcatcode, Ix] = sort(xcatcode, 'ascend');
sX = X(Ix,:);

fXcatcode = floor(sXcatcode); % floor
Jset = unique(fXcatcode);     % unique categories
J    = length(Jset); % # of unique category number

if J<max(r1,r2)
    disp('max(r1,r2) is larger than # of categories');
end


%% Factor estimation
X1 = sX(:,1:T1); %N*T1
X2 = sX(:,T1+1:T1+T2); %N*T2

% full sample
[f0,eig,f00] = svd(sX'*sX);
F_til       = f0(:,1:r2)*sqrt(T); %T*r2
lambda_til  = sX*F_til/T; %N*r1

% Pre-break
[f0,eig,f00] = svd(X1'*X1);
F1_til       = f0(:,1:r1)*sqrt(T1); %T1*r1
lambda1_til  = X1*F1_til/T1; %N*r1

% Post-break
[f0,eig,f00] = svd(X2'*X2);
F2_til       = f0(:,1:r2)*sqrt(T2); %T2*r2
lambda2_til  = X2*F2_til/T2; %N*r2


%% Factor identification (rotation) - Pre-break sample
temp_R2  = zeros(J,1); 
temp_bet = zeros(r1, J);

% loop over categories
for j=1:1:J
    jcat = Jset(j);
    
    xi  = X1(fXcatcode == jcat,:); % category j's data
    nxi = size(xi,1);
    
    % get the first pc for cat j
    [f0,eig,f00] = svd(xi'*xi);
    fxi          = f0(:,1)*sqrt(T1);
    lxi          = xi*fxi/T1;
    
    % OLS (without intercept)
    [bet, R2] = myols(fxi, F1_til);
    
    % store
    temp_bet(:,j) = bet;
    temp_R2(j)    = R2;
    
end

% Select factor rotation based on the largest R2
[sR2, Ix]  = sort(temp_R2, 'descend');
sbet       = temp_bet(:,Ix);
Omega1_til = sbet(1:r1,1:r1); % factor rotation for the pre-break sample

sJset    = Jset(Ix); % sort Jset as well
r2Jset   = sJset(r1+1:end); % kill cat that were used to construct F1
selJ     = sJset(1:r1); %selected categories

% Rotated factors and loadings
RF1_til      = F1_til*Omega1_til; % T1*r1
Rlambda1_til = (Omega1_til\lambda1_til')'; %n*r1



%% Factor identification (rotation) - Post-break sample
% step 1: X2' = Rlambda1_til*F12_til + resid
F12_til = myols(X2, Rlambda1_til)';

% step 2: F12_til = F2*C0 + resid
C0 = myols(F12_til, F2_til);

% step 3 (only when r2>r1): We can use the variables groups j that were not
% used to construct RF1_til.

nadd     = r2-r1; %# of additional factors
if nadd > 0
    
    temp_R2  = zeros(length(r2Jset),1);
    temp_bet = zeros(r2, length(r2Jset));
    
    for j = 1:1:length(r2Jset)
        jcat = r2Jset(j);
        
        xi  = X2(fXcatcode == jcat,:); % category j's data
        nxi = size(xi,1);
        
        % get the first pc for cat j
        [f0,eig,f00] = svd(xi'*xi);
        fxi          = f0(:,1)*sqrt(T2);
        lxi          = xi*fxi/T2;
        
        % OLS (without intercept)
        [bet, R2] = myols(fxi, F2_til);
        
        % store
        temp_bet(:,j) = bet;
        temp_R2(j)    = R2;
        
    end
    
    % Select factor rotation based on the largest R2
    [sR2, Ix]  = sort(temp_R2, 'descend');
    sbet       = temp_bet(:,Ix);
    Cadd       = sbet(:,1:nadd); % factor rotation for the pre-break sample
        
    temp_selJ = r2Jset(Ix);
    selJ      = [selJ; temp_selJ(1:nadd)]; %selected categories
end

% post-break rotation matrix
Omega2_til = [C0, Cadd];

% Rotated factors and loadings
RF2_til      = F2_til*Omega2_til; % T1*r1
Rlambda2_til = (Omega2_til\lambda2_til')'; %n*r1


%% Factor identification (rotation) - Full sample
temp_R2  = zeros(J,1); 
temp_bet = zeros(r2, J);

% loop over categories
for j=1:1:J
    jcat = Jset(j);
    
    xi  = sX(fXcatcode == jcat,:); % category j's data
    nxi = size(xi,1);
    
    % get the first pc for cat j
    [f0,eig,f00] = svd(xi'*xi);
    fxi          = f0(:,1)*sqrt(T);
    lxi          = xi*fxi/T;
    
    % OLS (without intercept)
    [bet, R2] = myols(fxi, F_til);
    
    % store
    temp_bet(:,j) = bet;
    temp_R2(j)    = R2;
    
end

% Select factor rotation based on the largest R2
[sR2, Ix]  = sort(temp_R2, 'descend');
sbet       = temp_bet(:,Ix);
Omega_til  = sbet(1:r2,1:r2); % factor rotation for the pre-break sample

selJfull   = Jset(Ix); % sort Jset as well

% Rotated factors and loadings
RF_til      = F_til*Omega_til; % T*r2
Rlambda_til = (Omega_til\lambda_til')'; %n*r2


%% Plot
plot(RF2_til(:,1:r2))
yyy = [RF1_til; RF2_til(:,1:r1)];

plot(yyy)


% plot(yyy(T1-5:T1+5,:));

%% Table
% split sample
tab0 = selJ; %selected cat number
tab1 = xcatname(selJ); %selected category names
tab2 = [ones(r1,1); 2*ones(r2-r1,1)]; % pre/post break identifier

tab  = [num2cell(tab0), tab1, num2cell(tab2)];

% full sample
tabf0 = num2cell(selJfull);
tabf1 = xcatname(selJfull);
tabf  = [tabf0, tabf1];

disp('-----------------------------------');
disp('break at');
disp(xcal(T1))

disp('split sample');
disp(tab)

disp('full sample');
disp(tabf)



